{-# LANGUAGE OverloadedStrings #-}

module Krobotkin.Muc where

import Krobotkin.Types
import Krobotkin.Monad

import Network.Xmpp
import Data.XML.Types

import Control.Monad.Reader

import qualified Data.Text as T

comeOnline :: BotMonad ()
comeOnline = do
   sess <- asks envXmppSess
   liftIO $ sendPresence presenceOnline sess >> return ()

joinMuc :: MucInfo -> BotMonad ()
joinMuc muc = do
   sess <- asks envXmppSess
   me <- liftIO $ getJid sess
   liftIO $ putStrLn $ "joining muc: " ++ show (mucJidNick muc)
   case me of
      Nothing -> liftIO $ putStrLn "error: failed to get jid from session"
      Just n -> do
         liftIO $ sendPresence
            (Presence
               { presenceID = Nothing
               , presenceFrom = Just n
               , presenceTo = Just $ mucJidNick muc
               , presenceLangTag = Nothing
               , presenceType = Available
               , presencePayload = [(
                  Element
                     { elementName = "{http://jabber.org/protocol/muc}x"
                     , elementAttributes = []
                     , elementNodes = []
                     }
                  )]
               , presenceAttributes = []
            })
            sess >> return ()

sendMucMessage :: T.Text -> Jid -> BotMonad ()
sendMucMessage content muc = do
   sess <- asks envXmppSess
   me <- liftIO $ getJid sess
   stid <- liftIO $ newStanzaID sess
   liftIO $ sendMessage
      (Message
         { messageID = Just stid
         , messageFrom = me
         , messageTo = Just muc
         , messageLangTag = Nothing
         , messageType = GroupChat
         , messageAttributes = []
         , messagePayload = [(
            Element {
               elementName = "body"
             , elementAttributes = []
             , elementNodes = [
                 (NodeContent $ ContentText content)
               ]
            }
         )]
      })
      sess >> return ()

getMucMessageBody :: Message -> Maybe T.Text
getMucMessageBody m =
   if elements == []
   then Nothing
   else case head (elementNodes $ head elements) of
      (NodeContent (ContentText content)) -> Just content
      _ -> Nothing
   where
      elements = filter (\p -> (nameLocalName (elementName p)) == "body") pl
      pl = messagePayload m
