{-# LANGUAGE OverloadedStrings #-}

module Krobotkin.Setup where

import Krobotkin.Options
import Krobotkin.Types

import Network.Xmpp

import Data.Maybe
import System.Random

import qualified Data.Text as T

-- TODO: have this report errors
createMucInfo :: Flags -> [MucInfo]
createMucInfo opts = catMaybes $ map
   (packMucInfo $ T.pack (flagUsername opts))
   (map T.pack (flagMucs opts))

packMucInfo :: T.Text -> T.Text -> Maybe MucInfo
packMucInfo username muc =
   case jidToTexts <$> jidFromText muc of
      Nothing -> Nothing
      Just (user, server, resource) ->
         let n = jidFromTexts user server $ Just $ T.replace "_" " " (fromMaybe username resource)
         in case n of
            Nothing -> Nothing
            Just o -> Just $ MucInfo (toBare o) o

createXmppSession :: Flags -> IO Session
createXmppSession opts = do
   sess <- session server login config
   case sess of
      Left e -> error $ "xmpp-failure: " ++ show e
      Right s -> return s
   where
      username = T.pack $ flagUsername opts
      password = T.pack $ flagPassword opts
      server = flagServer opts
      login = (Just (\_ -> ([scramSha1 username Nothing password]), Nothing))
      config = def { keepAlive = Just 60 }

createEnv :: Flags -> IO AppEnv
createEnv opts = do
   sess <- createXmppSession opts
   return $ AppEnv sess (createMucInfo opts)

createState :: IO AppState
createState = do
   rng <- getStdGen
   return $ AppState rng

